[][src]Struct arc_swap::cache::Cache

pub struct Cache<A, T> { /* fields omitted */ }

Caching handle for [ArcSwapAny].

Instead of loading (or leasing or something) the Arc on every request from the shared storage, this keeps another copy inside. Upon request it only cheaply revalidates it is up to date. If it is, access is significantly faster. If it is stale, the full load is done and the cache value is replaced. Under a read-heavy loads, the measured speedup are 10-25 times, depending on the architecture.

There are, however, downsides:

Examples

use std::sync::Arc;

use arc_swap::ArcSwap;
use arc_swap::cache::Cache;

let shared = Arc::new(ArcSwap::from_pointee(42));
// Start 10 worker threads...
for _ in 0..10 {
    let mut cache = Cache::new(Arc::clone(&shared));
    std::thread::spawn(move || {
        // Keep loading it like mad..
        loop {
            let value = cache.load();
            do_something(value);
        }
    });
}
shared.store(Arc::new(12));

Methods

impl<A, T, S> Cache<A, T> where
    A: Deref<Target = ArcSwapAny<T, S>>,
    T: RefCnt,
    S: LockStorage
[src]

pub fn new(arc_swap: A) -> Self[src]

Creates a new caching handle.

The parameter is something dereferencing into an [ArcSwapAny] (eg. either to ArcSwap or ArcSwapOption). That can be [ArcSwapAny] itself, but that's not very useful. But it also can be a reference to it or Arc, which makes it possible to share the [ArcSwapAny] with multiple caches or access it in non-cached way too.

pub fn arc_swap(&self) -> &A::Target[src]

Gives access to the (possibly shared) cached [ArcSwapAny].

pub fn load(&mut self) -> &T[src]

Loads the currently held value.

This first checks if the cached value is up to date. This check is very cheap.

If it is up to date, the cached value is simply returned without additional costs. If it is outdated, a load is done on the underlying shared storage. The newly loaded value is then stored in the cache and returned.